/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.applicationprocessor.processor;

import java.util.List;

import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.exceptionhandler.ExceptionHandler;
import org.tinygroup.exceptionhandler.ExceptionHandlerManager;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.parser.filter.NameFilter;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.xmlparser.node.XmlNode;

public class ExceptionProcessor implements ApplicationProcessor {
	private ExceptionHandlerManager manager;
	private XmlNode xmlNode;
	private static Logger logger = LoggerFactory.getLogger(ExceptionProcessor.class);
	public ExceptionHandlerManager getManager() {
		return manager;
	}

	public void setManager(ExceptionHandlerManager manager) {
		this.manager = manager;
	}

	public String getNodeName() {
		return "exception-config";
	}

	public void setConfiguration(XmlNode xmlNode) {
		logger.logMessage(LogLevel.INFO, "读取exception-handler配置");
		this.xmlNode = xmlNode;
		parse(xmlNode);
		logger.logMessage(LogLevel.INFO, "读取exception-handler配置完毕");
	}

	private void parse(XmlNode xmlNode) {
		if(xmlNode==null){
			logger.logMessage(LogLevel.INFO, "exception-handler配置为空");
			return;
		}
		NameFilter<XmlNode> nameFilter = new NameFilter<XmlNode>(xmlNode);
		List<XmlNode> handlerList = nameFilter.findNodeList("exception-handler");
		for(XmlNode handler:handlerList){
			String exception = handler.getAttribute("exception");
			String handlerBean = handler.getAttribute("handler");
			logger.logMessage(LogLevel.INFO, "添加exception-handler,Exception:{0},handerBean:{1}",exception,handlerBean);
			ExceptionHandler<?> exceptionHandler = SpringUtil.getBean(handlerBean);
			try {
				manager.addHandler(exception, exceptionHandler);
			} catch (ClassNotFoundException e) {
				logger.logMessage(LogLevel.INFO, "添加exception-handler出错,Exception类:{0}找不到",exception,handlerBean);
				continue;
			}
			logger.logMessage(LogLevel.INFO, "添加exception-handler,Exception:{0},handerBean:{1}完成",exception,handlerBean);
		}
		
	}

	public XmlNode getConfiguration() {
		return xmlNode;
	}

	public void start() {

	}

	public void stop() {

	}

	public void setApplication(Application application) {

	}

}
