/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.applicationprocessor.processor;

import org.tinygroup.application.Application;
import org.tinygroup.application.ApplicationProcessor;
import org.tinygroup.fileresolver.FileResolver;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.springutil.SpringUtil;
import org.tinygroup.xmlparser.node.XmlNode;

/**
 * 加载处理器，并执行加载
 * 
 * @author luoguo
 * 
 */
public class FileMonitorProcessor implements ApplicationProcessor {
	private static final int DEFAULT_INTERVAL = 5;

	private static final String FILE_MONITOR_NODE_NAME = "file-monitor";

	private static Logger logger = LoggerFactory
			.getLogger(FileMonitorProcessor.class);

	private FileMonitorThread thread = null;
	private int interval = DEFAULT_INTERVAL;
	private boolean enable = false;
	private XmlNode fileMonitor;


	public void setConfiguration(XmlNode fileMonitor) {
		this.fileMonitor = fileMonitor;
		if (fileMonitor != null) {
			String strInterrupt = fileMonitor.getAttribute("interval");
			if (strInterrupt != null && strInterrupt.length() > 0) {
				interval = Integer.parseInt(strInterrupt);
			}
			String enableString = fileMonitor.getAttribute("enable");
			if (enableString != null && enableString.length() > 0) {
				enable = Boolean.valueOf(enableString);
			}
		}
	}

	public void start() {
		if (enable) {
			if (thread != null) {
				thread.stop = true;
			}
			thread = new FileMonitorThread();
			thread.run();
		}
	}

	public void stop() {
		if (enable && thread != null && thread.isAlive()) {
			thread.stop = true;
		}
	}

	private class FileMonitorThread extends Thread {
		private static final int MILLISECOND_PER_SECOND = 1000;
		private volatile boolean stop = false;

		public void run() {
			while (!stop) {
				try {
					sleep(interval * MILLISECOND_PER_SECOND);
					logger.logMessage(LogLevel.INFO, "定时扫描文件变化......");
					FileResolver resolver = SpringUtil.getBean("fileResolver");
					resolver.refresh();
					logger.logMessage(LogLevel.INFO, "定时扫描文件结束.");
				} catch (InterruptedException e) {
					logger.errorMessage(e.getMessage(), e);
				}
			}
		}
	}

	public String getNodeName() {
		return FILE_MONITOR_NODE_NAME;
	}

	public XmlNode getConfiguration() {
		return fileMonitor;
	}

	public void setApplication(Application application) {

	}
}
